import random
import typing
import urllib.parse
from datetime import datetime
from timeit import default_timer as timer

import sqlalchemy
from sqlalchemy import (
    Column,
    Table,
    MetaData,
    create_engine,
    DateTime,
    func,
    literal_column,
)
from bakplane.bakplane_pb2 import (
    ResolveResourcePathResponse,
    ResolveResourceIntentResponse,
    ComparisonType,
)
from bakplane.extensions.base import (
    WriteResponse,
    WriteRequest,
    ReadResponse,
    ReadAllRequest,
    ReadRequest,
    ReadRequestBuildingBlocks,
    DataStoreHelper,
    INFINITY_PROXY_DATE,
    DT_COLUMN_NAME,
    DEFAULT_SERIES,
)
from bakplane.extensions.spark.utils import (
    to_spark_save_mode,
    timestamp_to_date_string,
    datetime_to_date_string,
)
from bakplane.utils import to_execution_statistics


class JDBCStoreHelper(DataStoreHelper):
    def __init__(
        self,
        user: str,
        password: str,
        host: str,
        port: int,
        database: str,
        schema_name: str,
        driver: str,
        table_prefixes: typing.List[str] = None,
        series_schema: str = "series",
    ):

        self.metadata = MetaData()
        self.engine = create_engine(
            f"{driver}://{user}:{password}@{host}:{port}/{database}",
            executemany_values_page_size=10000,
            executemany_batch_page_size=500,
            executemany_mode="batch",
        )
        self.schema_name = schema_name
        self.table_prefixes = table_prefixes
        self.series_schema = series_schema
        self.frequency_to_tbl = {}

        self.__preload_existing_series()

    def get_series_table(
        self, series: typing.Tuple[datetime, datetime, str], alias: str = None
    ) -> Table:
        freq = series[2].strip().lower()

        if freq not in self.frequency_to_tbl:
            raise RuntimeError(
                f"Frequency `{freq}` is not found within cache. Verify the series exist in `{self.series_schema} on `{self.engine.url}`."
            )

        if alias is None:
            alias = f"t_{freq}_{str(int(datetime.utcnow().timestamp()))}_{str(random.randint(1, 100000))}"

        return self.frequency_to_tbl[freq].alias(alias)

    def __preload_existing_series(self):
        for s in DEFAULT_SERIES:

            freq = s.lower().strip()
            tbl_name = f"s_{freq}"

            columns = [
                Column(
                    DT_COLUMN_NAME, DateTime, nullable=False, primary_key=True
                )
            ]

            tbl = Table(
                tbl_name, self.metadata, *columns, schema=self.series_schema
            )

            if self.engine.dialect.has_table(
                self.engine, table_name=tbl_name, schema=self.series_schema
            ):
                self.frequency_to_tbl[freq] = tbl

    def create_temporary_table(
        self,
        prefix: str,
        columns: typing.List[Column],
        rows: typing.List[typing.Dict[str, typing.Any]],
    ) -> Table:

        for c in columns:
            c.table = None

        tbl = Table(
            prefix
            + "_"
            + str(int(datetime.utcnow().timestamp()))
            + "_"
            + str(random.randint(1, 1000)),
            self.metadata,
            *columns,
            schema=self.schema_name,
        )

        tbl.create(bind=self.engine)
        self.engine.execute(tbl.insert(), rows)

        return tbl


def __build_read_all_query(
    r: ReadAllRequest, res: ResolveResourcePathResponse
) -> str:
    return f'select {", ".join(r.columns)} from {res.context["schema_name"]}.{res.context["table_name"]}'


def __get_range_semantics_ops(semantics: str) -> typing.Tuple[str, str]:
    if semantics == "[)":
        return ">=", "<"
    elif semantics == "[]":
        return ">=", "<="
    elif semantics == "(]":
        return ">", "<="
    elif semantics == "()":
        return ">", "<"
    else:
        raise RuntimeError(f"`{semantics}` is not a valid range semantics.")


def __add_effective_dating_constraint(
    blocks: ReadRequestBuildingBlocks,
    res: ResolveResourceIntentResponse,
    r: ReadRequest,
):
    a = blocks.asset_table

    if r.effective_dating is None:
        return

    op1, op2 = __get_range_semantics_ops(
        blocks.effective_dating_range_semantics
    )

    if blocks.effective_dating_table is not None:
        blocks.constraints.append(
            blocks.effective_dating_table.c.dt.op(op1)(a.c.effective_start_dt)
        )

        blocks.constraints.append(
            blocks.effective_dating_table.c.dt.op(op2)(a.c.effective_end_dt)
        )

        blocks.constraints.append(
            blocks.effective_dating_table.c.dt.op(op1)(
                datetime_to_date_string(blocks.effective_start_dt)
            )
        )

        blocks.constraints.append(
            blocks.effective_dating_table.c.dt.op(op2)(
                datetime_to_date_string(blocks.effective_end_dt)
            )
        )

        blocks.columns.append(
            blocks.effective_dating_table.c.dt.label("effective_dt")
        )

    else:
        blocks.constraints.append(
            a.c.effective_start_dt
            <= timestamp_to_date_string(res.effective_dating.end_dt.seconds)
        )
        blocks.constraints.append(
            a.c.effective_end_dt
            >= timestamp_to_date_string(res.effective_dating.start_dt.seconds)
        )


def __add_valid_dating_constraint(
    blocks: ReadRequestBuildingBlocks, res: ResolveResourceIntentResponse
):
    a = blocks.asset_table

    op1, op2 = __get_range_semantics_ops(blocks.valid_dating_range_semantics)

    if blocks.valid_dating_table is not None:
        blocks.constraints.append(
            blocks.valid_dating_table.c.dt.op(op1)(a.c.valid_start_dt)
        )

        blocks.constraints.append(
            blocks.valid_dating_table.c.dt.op(op2)(a.c.valid_end_dt)
        )

        blocks.constraints.append(
            blocks.valid_dating_table.c.dt.op(op1)(
                datetime_to_date_string(blocks.valid_start_dt)
            )
        )

        blocks.constraints.append(
            blocks.valid_dating_table.c.dt.op(op2)(
                datetime_to_date_string(blocks.valid_end_dt)
            )
        )

        blocks.columns.append(blocks.valid_dating_table.c.dt.label("valid_dt"))


def __populate_building_blocks_dating(
    request_field: str,
    start_field: str,
    end_field: str,
    dating_tbl_field: str,
    semantics_field: str,
    helper: DataStoreHelper,
    blocks: ReadRequestBuildingBlocks,
    r: ReadRequest,
):
    if not hasattr(r, request_field):
        raise RuntimeError("Invalid field name.")

    dating = getattr(r, request_field)

    if dating is None:
        return

    if isinstance(dating, tuple):
        if len(dating) == 2:
            start, end = dating

            setattr(blocks, start_field, start)
            setattr(blocks, end_field, end)

        elif len(dating) == 3:
            setattr(
                blocks,
                dating_tbl_field,
                helper.get_series_table(dating, alias=request_field),
            )
            setattr(blocks, start_field, dating[0])
            setattr(blocks, end_field, dating[1])
        elif len(dating) == 4:
            setattr(
                blocks,
                dating_tbl_field,
                helper.get_series_table(dating, alias=request_field),
            )
            setattr(blocks, start_field, dating[0])
            setattr(blocks, end_field, dating[1])
            setattr(blocks, semantics_field, dating[3])

        else:
            raise RuntimeError("Invalid dating constraint.")

    elif isinstance(dating, datetime):
        setattr(blocks, start_field, dating)
        setattr(blocks, end_field, INFINITY_PROXY_DATE)
    else:
        raise ValueError("Effective dating has unknown type.")


def __build_read_query(
    r: ReadRequest,
    res: ResolveResourceIntentResponse,
    dialect,
    store_helper: DataStoreHelper,
) -> str:
    from sqlalchemy import (
        Table,
        MetaData,
        Column,
        select,
        and_,
    )

    if (r.columns is None or len(r.columns) <= 0) and r.query_builder is None:
        raise RuntimeError("You must provide a set of columns.")

    metadata: MetaData = MetaData()

    src_tbl: Table = Table(
        res.table_name,
        metadata,
        *[Column(x.name) for x in res.columns],
        schema=res.schema_name,
    )

    mapping_tbl: Table = Table(
        res.mapping.table_name,
        metadata,
        *[Column(x.name) for x in res.mapping.columns],
        schema=res.mapping.schema_name,
    )

    a = src_tbl.alias("a")

    mapping_columns = []
    for column in res.mapping.columns:
        if not column.is_effective_dating:
            mapping_columns.append(
                mapping_tbl.c.get(column.name).label(column.name)
            )

    m = mapping_tbl.alias("m")
    clauses = []
    for clause in res.mapping.clauses:
        if clause.comparison == ComparisonType.EQ:
            clauses.append(
                m.c.get(clause.mapping_field_name)
                == a.c.get(clause.resource_field_name)
            )

    blocks = ReadRequestBuildingBlocks(
        mapping_table=m,
        asset_table=a,
        selectable=m.join(a, and_(*clauses)),
        knowledge_dt=r.knowledge_dt,
        constraints=[
            and_(
                *[
                    datetime_to_date_string(r.knowledge_dt)
                    >= a.c.knowledge_start_dt,
                    datetime_to_date_string(r.knowledge_dt)
                    < a.c.knowledge_end_dt,
                ]
            )
        ],
        valid_dating_range_semantics="[)",
        effective_dating_range_semantics="[)",
    )

    __populate_building_blocks_dating(
        "effective_dating",
        "effective_start_dt",
        "effective_end_dt",
        "effective_dating_table",
        "effective_dating_range_semantics",
        store_helper,
        blocks,
        r,
    )

    __populate_building_blocks_dating(
        "valid_dating",
        "valid_start_dt",
        "valid_end_dt",
        "valid_dating_table",
        "valid_dating_range_semantics",
        store_helper,
        blocks,
        r,
    )

    if r.query_builder:
        res = r.query_builder(blocks)
    else:
        blocks.columns = [a.c.get(x) for x in r.columns if x in a.c]

        if r.partition_num is not None:
            if r.partition_column_name not in a.c:
                raise RuntimeError("Could not identify partition column.")

            index_expression = func.abs(a.c.knowledge_hash % r.partition_num)
            blocks.columns.append(index_expression.label("partition"))

        if r.additional_constraints is not None:
            blocks.constraints.append(r.additional_constraints(blocks))

        if r.effective_dating is not None:
            if r.effective_dating_hint is None:
                blocks.constraints.append(
                    a.c.effective_start_dt
                    <= blocks.mapping_table.c.effective_end_dt
                )

                blocks.constraints.append(
                    a.c.effective_end_dt
                    >= blocks.mapping_table.c.effective_start_dt
                )
        else:
            blocks.constraints.append(
                a.c.valid_start_dt <= blocks.mapping_table.c.effective_end_dt
            )

            blocks.constraints.append(
                a.c.valid_end_dt >= blocks.mapping_table.c.effective_start_dt
            )

        __add_valid_dating_constraint(blocks, res)

        if r.effective_dating_hint is None:
            __add_effective_dating_constraint(blocks, res, r)

        else:
            if isinstance(r.effective_dating_hint, tuple):
                s = datetime_to_date_string(
                    typing.cast(datetime, r.effective_dating_hint[0])
                )
                e = datetime_to_date_string(
                    typing.cast(datetime, r.effective_dating_hint[1])
                )

                blocks.constraints.append(a.c.effective_start_dt <= e)
                blocks.constraints.append(a.c.effective_end_dt >= s)
            elif isinstance(r.effective_dating_hint, str):
                col = typing.cast(str, r.effective_dating_hint)

                s = col + "_start_dt"
                e = col + "_end_dt"

                blocks.constraints.append(a.c.effective_start_dt <= m.c.get(e))
                blocks.constraints.append(a.c.effective_end_dt >= m.c.get(s))
            else:
                raise RuntimeError("Invalid type for effective_dating_hint.")

        if r.modifiers:
            for m in r.modifiers:
                blocks = m.execute(blocks, store_helper)

        res = (
            select(blocks.columns)
            .select_from(blocks.selectable)
            .where(and_(*blocks.constraints))
        )

    query = str(
        res.compile(
            dialect=dialect,
            compile_kwargs={"literal_binds": True},
        )
    )

    return query.replace("%%", "%")


def write_fn(
    r: WriteRequest, res: ResolveResourcePathResponse, driver: str, url: str
) -> WriteResponse:
    start = timer()

    r.df.write.format("jdbc").mode(to_spark_save_mode(r.mode)).option(
        "driver", driver
    ).option("url", url).option(
        "dbtable", f"{res.context['schema_name']}.{res.context['table_name']}"
    ).option(
        "user", res.context["username"]
    ).option(
        "password", urllib.parse.unquote(res.context["password"])
    ).save()

    end = timer()

    return WriteResponse(
        execution_statistics=to_execution_statistics(start, end)
    )


def __read_query(
    spark,
    url: str,
    driver: str,
    query: str,
    user: str,
    password: str,
    partition_num: int,
):
    start = timer()

    if partition_num is None:
        df = (
            spark.read.format("jdbc")
            .option("url", url)
            .option("query", query)
            .option("user", user)
            .option("password", urllib.parse.unquote(password))
            .option("driver", driver)
            .load()
        )
    else:
        df = (
            spark.read.format("jdbc")
            .option("url", url)
            .option("dbtable", f"({query}) s")
            .option("user", user)
            .option("password", urllib.parse.unquote(password))
            .option("driver", driver)
            .option("partitionColumn", "partition")
            .option("lowerBound", 0)
            .option("upperBound", partition_num - 1)
            .option("numPartitions", partition_num)
            .load()
        )

    end = timer()

    return ReadResponse(
        df=df,
        execution_statistics=to_execution_statistics(start, end),
        context={
            "query": query,
        },
    )


def read_all_fn(
    ctx: typing.Any,
    r: ReadAllRequest,
    res: ResolveResourcePathResponse,
    url: str,
    driver: str,
) -> ReadResponse:
    return __read_query(
        spark=ctx,
        url=url,
        driver=driver,
        query=__build_read_all_query(r, res),
        user=res.context["username"],
        password=res.context["password"],
        partition_num=r.partition_num,
    )


def read_fn(
    ctx: typing.Any,
    r: ReadRequest,
    res: ResolveResourceIntentResponse,
    dialect,
    url: str,
    driver: str,
    store_helper: DataStoreHelper,
) -> ReadResponse:
    return __read_query(
        spark=ctx,
        url=url,
        driver=driver,
        query=__build_read_query(r, res, dialect, store_helper),
        user=res.context["username"],
        password=res.context["password"],
        partition_num=r.partition_num,
    )



import certifi
import grpc

from bakplane.bakplane_pb2_grpc import BakplaneStub


class BakplaneClient(object):
    def __init__(self, address: str, secure: bool = True):
        self.address = address
        self.secure = secure

    def get_connection(self) -> BakplaneStub:
        if not self.secure:
            return BakplaneStub(grpc.insecure_channel(self.address))

        with open(certifi.where(), "rb") as certs:
            credentials = grpc.ssl_channel_credentials(certs.read())
            return BakplaneStub(
                grpc.secure_channel(self.address, credentials=credentials)
            )



import typing

import pandas as pd
from sqlalchemy import Column, DateTime, and_, func, select

from bakplane.extensions.base import (
    ReadRequestBuildingBlocks,
    Modifier,
    DataStoreHelper,
)


class NearestModifier(Modifier):
    NEAREST_COLUMN_NAME = "nearest"

    def __init__(
        self,
        n: int,
        partition_fields: typing.List[str],
        ordered_fields: typing.List[typing.Tuple[str, str]],
    ):
        self.partition_fields = partition_fields
        self.ordered_fields = ordered_fields
        self.n = n

    def execute(
        self, r: ReadRequestBuildingBlocks, h: DataStoreHelper
    ) -> ReadRequestBuildingBlocks:

        if r.modifier_context and "frequency" in r.modifier_context:
            frequency_tbl = r.modifier_context["frequency"]["table"]
            column_name = FrequencyModifier.FREQUENCY_COLUMN_NAME
        elif r.effective_dating_table is not None:
            frequency_tbl = r.effective_dating_table
            column_name = "dt"
        elif r.valid_dating_table is not None:
            frequency_tbl = r.valid_dating_table
            column_name = "dt"
        else:
            raise ValueError(
                "Building blocks do not contain a frequency table."
            )

        p = [r.asset_table.c.get(x) for x in self.partition_fields] + [
            frequency_tbl.c.get(column_name)
        ]
        o = [
            r.asset_table.c.get(x[0]).desc()
            if x[1] == "desc"
            else r.asset_table.c.get(x[0]).desc()
            for x in self.ordered_fields
        ]

        r.columns.append(
            func.row_number()
            .over(partition_by=p, order_by=o)
            .label(NearestModifier.NEAREST_COLUMN_NAME)
        )
        cte = (
            select(r.columns)
            .select_from(r.selectable)
            .where(and_(*r.constraints))
            .cte("nearest")
        )

        r.selectable = cte
        r.columns = [cte.c.get(x.name) for x in r.columns]
        r.constraints = [
            cte.c.get(NearestModifier.NEAREST_COLUMN_NAME) <= self.n
        ]

        return r


class FrequencyModifier(Modifier):
    FREQUENCY_COLUMN_NAME = "frequency_dt"
    PREFIX_NAME = "t_frequency"

    def __init__(self, frequency: str):
        self.frequency = frequency

    def execute(
        self, r: ReadRequestBuildingBlocks, h: DataStoreHelper
    ) -> ReadRequestBuildingBlocks:

        if r.effective_dating_table is not None:
            raise ValueError(
                "A frequency modifier cannot be applied to non range queries."
            )

        ranges = pd.bdate_range(
            start=r.effective_start_dt,
            end=r.effective_end_dt,
            freq=self.frequency,
        )

        tbl = h.create_temporary_table(
            prefix=FrequencyModifier.PREFIX_NAME,
            columns=[
                Column(
                    FrequencyModifier.FREQUENCY_COLUMN_NAME,
                    DateTime,
                    nullable=False,
                )
            ],
            rows=[
                {FrequencyModifier.FREQUENCY_COLUMN_NAME: x.to_pydatetime()}
                for x in ranges
            ],
        )

        r.selectable = r.selectable.join(
            tbl,
            and_(
                tbl.c.get(FrequencyModifier.FREQUENCY_COLUMN_NAME)
                >= r.asset_table.c.effective_start_dt,
                tbl.c.get(FrequencyModifier.FREQUENCY_COLUMN_NAME)
                < r.asset_table.c.effective_end_dt,
            ),
        )

        r.columns.append(tbl.c.get(FrequencyModifier.FREQUENCY_COLUMN_NAME))

        if r.modifier_context is None:
            r.modifier_context = {}

        r.modifier_context["frequency"] = {"table": tbl}

        return r



import logging
import typing
import traceback


from datetime import datetime, timedelta
from timeit import default_timer as timer

from bakplane.bakplane_pb2 import *
from bakplane.bakplane_pb2_grpc import BakplaneStub
from bakplane.errors import BakplaneException
from bakplane.extensions.base import (
    BaseExtension,
    PluginEntry,
    ReadRequest,
    ReadResponse,
    WriteRequest,
    WriteResponse,
    ReadUniverseRequest,
    ReadUniverseResponse,
    ReadAllRequest,
    ExecutionStatistics,
)
from bakplane.extensions.spark.stores import redshift, postgres
from bakplane.utils import to_proto_timestamp_range, parse_value_by_type

__plugins = {}


def register_plugin(entry):
    if entry.code not in __plugins:
        __plugins[entry.code] = entry


def get_plugin_or_die(code: str) -> PluginEntry:
    if code not in __plugins:
        raise RuntimeError(f"Failed to obtain plugin `{code}`.")

    return __plugins[code]


class SparkExtension(BaseExtension):
    def __init__(self, stub: BakplaneStub, spark):
        self.stub = stub
        self.spark = spark

    def read(self, r: ReadRequest) -> ReadResponse:
        try:
            intent = self.stub.ResolveResourceIntent(
                ResolveResourceRequest(
                    universe=r.universe,
                    effective_dating=to_proto_timestamp_range(
                        r.get_derived_effective_start_dt(),
                        r.get_derived_effective_end_dt(),
                    ),
                    path=r.path,
                    warehouse=r.warehouse,
                    pointers=r.pointers,
                )
            )

            if not intent:
                raise RuntimeError("Failed to resolve intent.")

            if r.knowledge_dt is None:
                tomorrow_dt = datetime.utcnow() + timedelta(days=1)
                r.knowledge_dt = datetime(
                    year=tomorrow_dt.year,
                    month=tomorrow_dt.month,
                    day=tomorrow_dt.day,
                )

            if r.effective_dating is None and r.valid_dating is None:
                raise RuntimeError(
                    "You must provide at least effective_dating or valid_dating."
                )

            p = get_plugin_or_die(intent.warehouse_kind)

            return p.read_fn(self.spark, r, intent)
        except Exception as ex:
            raise BakplaneException(
                "Failed to execute read request due to: "
                + traceback.format_exc(),
                ex,
            )

    def write(self, r: WriteRequest) -> WriteResponse:
        try:
            logging.info(f"Writing to resource path `{r.path}`.")

            internals = self.stub.ResolveResourcePath(
                ResolveResourcePathRequest(path=r.path, warehouse=r.warehouse)
            )

            if not internals:
                raise RuntimeError("Failed to resolve resource path.")

            p = get_plugin_or_die(internals.warehouse_kind)
            return p.write_fn(r, internals)
        except Exception as ex:
            raise BakplaneException("Failed to execute write request.", ex)

    def read_universe(self, r: ReadUniverseRequest) -> ReadUniverseResponse:
        try:
            logging.info(f"Resolving universe `{r}`.")

            res = self.stub.ResolveUniverse(
                ResolveUniverseRequest(
                    universe=r.universe,
                    effective_dating=to_proto_timestamp_range(
                        r.effective_start_dt, r.effective_end_dt
                    ),
                )
            )

            if not res:
                raise RuntimeError("Failed to resolve universe.")
            return self.__build_read_universe_response(res)

        except Exception as ex:
            raise BakplaneException("Failed to read universe.", ex)

    def read_all(self, r: ReadAllRequest) -> ReadResponse:
        try:
            logging.info(f"Resolving resource path `{r.path}`.")

            res = self.stub.ResolveResourcePath(
                ResolveResourcePathRequest(path=r.path, warehouse=r.warehouse)
            )

            if not res:
                raise RuntimeError("Failed to resolve resource path.")

            p = get_plugin_or_die(res.warehouse_kind)
            return p.read_all_fn(self.spark, r, res)
        except Exception as ex:
            raise BakplaneException("Failed to execute read all request.", ex)

    def __build_read_universe_response(
        self, res: typing.Iterable[ResolveUniverseGroupEntry]
    ) -> ReadUniverseResponse:
        start = timer()

        rows = []
        schema = []
        ix = 0

        for g in res:
            row = []
            for e in g.entries:
                if ix == 0:
                    schema.append(e.attribute)
                row.append(parse_value_by_type(e.value, e.data_type))

            ix += 1
            rows.append(row)

        end = timer()

        return ReadUniverseResponse(
            df=self.spark.createDataFrame(data=rows, schema=schema),
            execution_statistics=ExecutionStatistics(elapsed_ms=(end - start)),
        )


register_plugin(redshift.get_registration())
register_plugin(postgres.get_registration())



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from bakplane import bakplane_pb2 as bakplane_dot_bakplane__pb2


class BakplaneStub(object):
    """
    Service for manging the lifecycle of ingestion, mastering, warehousing, and querying

    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ApplyAdapter = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyAdapter",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyAdapterRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyAdapterResponse.FromString,
        )
        self.RetractAdapter = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractAdapter",
            request_serializer=bakplane_dot_bakplane__pb2.RetractAdapterRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractAdapterResponse.FromString,
        )
        self.ListAdapter = channel.unary_unary(
            "/io.openaristos.Bakplane/ListAdapter",
            request_serializer=bakplane_dot_bakplane__pb2.ListAdapterRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListAdapterResponse.FromString,
        )
        self.EnableAdapter = channel.unary_unary(
            "/io.openaristos.Bakplane/EnableAdapter",
            request_serializer=bakplane_dot_bakplane__pb2.EnableAdapterRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.EnableAdapterResponse.FromString,
        )
        self.DisableAdapter = channel.unary_unary(
            "/io.openaristos.Bakplane/DisableAdapter",
            request_serializer=bakplane_dot_bakplane__pb2.DisableAdapterRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.DisableAdapterResponse.FromString,
        )
        self.ApplyResource = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyResource",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyResourceResponse.FromString,
        )
        self.RetractResource = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractResource",
            request_serializer=bakplane_dot_bakplane__pb2.RetractResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractResourceResponse.FromString,
        )
        self.ListResources = channel.unary_unary(
            "/io.openaristos.Bakplane/ListResources",
            request_serializer=bakplane_dot_bakplane__pb2.ListResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListResourceResponse.FromString,
        )
        self.EnableResource = channel.unary_unary(
            "/io.openaristos.Bakplane/EnableResource",
            request_serializer=bakplane_dot_bakplane__pb2.EnableResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.EnableResourceResponse.FromString,
        )
        self.PreviewResource = channel.unary_unary(
            "/io.openaristos.Bakplane/PreviewResource",
            request_serializer=bakplane_dot_bakplane__pb2.PreviewResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.PreviewResourceResponse.FromString,
        )
        self.DisableResource = channel.unary_unary(
            "/io.openaristos.Bakplane/DisableResource",
            request_serializer=bakplane_dot_bakplane__pb2.DisableResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.DisableResourceResponse.FromString,
        )
        self.MigrateResource = channel.unary_unary(
            "/io.openaristos.Bakplane/MigrateResource",
            request_serializer=bakplane_dot_bakplane__pb2.MigrateResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.MigrateResourceResponse.FromString,
        )
        self.MigrateAllResources = channel.unary_unary(
            "/io.openaristos.Bakplane/MigrateAllResources",
            request_serializer=bakplane_dot_bakplane__pb2.MigrateAllResourcesRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.MigrateAllResourcesResponse.FromString,
        )
        self.ResourceMigrationStatus = channel.unary_unary(
            "/io.openaristos.Bakplane/ResourceMigrationStatus",
            request_serializer=bakplane_dot_bakplane__pb2.MigrationStatusRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.MigrateResourceResponse.FromString,
        )
        self.ResolveResource = channel.unary_stream(
            "/io.openaristos.Bakplane/ResolveResource",
            request_serializer=bakplane_dot_bakplane__pb2.ResolveResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ResolveResourceResponse.FromString,
        )
        self.ResolveResourceIntent = channel.unary_unary(
            "/io.openaristos.Bakplane/ResolveResourceIntent",
            request_serializer=bakplane_dot_bakplane__pb2.ResolveResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ResolveResourceIntentResponse.FromString,
        )
        self.ResolveQueryResource = channel.unary_unary(
            "/io.openaristos.Bakplane/ResolveQueryResource",
            request_serializer=bakplane_dot_bakplane__pb2.ResolveQueryResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ResolveQueryResourceResponse.FromString,
        )
        self.ResolveUniverse = channel.unary_stream(
            "/io.openaristos.Bakplane/ResolveUniverse",
            request_serializer=bakplane_dot_bakplane__pb2.ResolveUniverseRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ResolveUniverseGroupEntry.FromString,
        )
        self.ApplyLandingZone = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyLandingZone",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyLandingZoneRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyLandingZoneResponse.FromString,
        )
        self.RetractLandingZone = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractLandingZone",
            request_serializer=bakplane_dot_bakplane__pb2.RetractLandingZoneRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractLandingZoneResponse.FromString,
        )
        self.ListLandingZone = channel.unary_unary(
            "/io.openaristos.Bakplane/ListLandingZone",
            request_serializer=bakplane_dot_bakplane__pb2.ListLandingZoneRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListLandingZoneResponse.FromString,
        )
        self.EnableLandingZone = channel.unary_unary(
            "/io.openaristos.Bakplane/EnableLandingZone",
            request_serializer=bakplane_dot_bakplane__pb2.EnableLandingZoneRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.EnableLandingZoneResponse.FromString,
        )
        self.DisableLandingZone = channel.unary_unary(
            "/io.openaristos.Bakplane/DisableLandingZone",
            request_serializer=bakplane_dot_bakplane__pb2.DisableLandingZoneRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.DisableLandingZoneResponse.FromString,
        )
        self.ApplyOntology = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyOntology",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyOntologyRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyOntologyResponse.FromString,
        )
        self.RetractOntology = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractOntology",
            request_serializer=bakplane_dot_bakplane__pb2.RetractOntologyRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractOntologyResponse.FromString,
        )
        self.ListOntology = channel.unary_unary(
            "/io.openaristos.Bakplane/ListOntology",
            request_serializer=bakplane_dot_bakplane__pb2.ListOntologyRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListOntologyResponse.FromString,
        )
        self.ApplyExecutionEngine = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyExecutionEngine",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyExecutionEngineRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyExecutionEngineResponse.FromString,
        )
        self.RetractExecutionEngine = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractExecutionEngine",
            request_serializer=bakplane_dot_bakplane__pb2.RetractExecutionEngineRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractExecutionEngineResponse.FromString,
        )
        self.ListExecutionEngine = channel.unary_unary(
            "/io.openaristos.Bakplane/ListExecutionEngine",
            request_serializer=bakplane_dot_bakplane__pb2.ListExecutionEngineRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListExecutionEngineResponse.FromString,
        )
        self.EnableExecutionEngine = channel.unary_unary(
            "/io.openaristos.Bakplane/EnableExecutionEngine",
            request_serializer=bakplane_dot_bakplane__pb2.EnableExecutionEngineRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.EnableExecutionEngineResponse.FromString,
        )
        self.DisableExecutionEngine = channel.unary_unary(
            "/io.openaristos.Bakplane/DisableExecutionEngine",
            request_serializer=bakplane_dot_bakplane__pb2.DisableExecutionEngineRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.DisableExecutionEngineResponse.FromString,
        )
        self.ApplyWarehouse = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyWarehouse",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyWarehouseRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyWarehouseResponse.FromString,
        )
        self.ApplyValidator = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyValidator",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyValidatorRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyValidatorResponse.FromString,
        )
        self.RetractValidator = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractValidator",
            request_serializer=bakplane_dot_bakplane__pb2.RetractValidatorRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractValidatorResponse.FromString,
        )
        self.ListValidator = channel.unary_unary(
            "/io.openaristos.Bakplane/ListValidator",
            request_serializer=bakplane_dot_bakplane__pb2.ListValidatorRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListValidatorResponse.FromString,
        )
        self.RetractWarehouse = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractWarehouse",
            request_serializer=bakplane_dot_bakplane__pb2.RetractWarehouseRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractWarehouseResponse.FromString,
        )
        self.ListWarehouse = channel.unary_unary(
            "/io.openaristos.Bakplane/ListWarehouse",
            request_serializer=bakplane_dot_bakplane__pb2.ListWarehouseRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListWarehouseResponse.FromString,
        )
        self.EnableWarehouse = channel.unary_unary(
            "/io.openaristos.Bakplane/EnableWarehouse",
            request_serializer=bakplane_dot_bakplane__pb2.EnableWarehouseRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.EnableWarehouseResponse.FromString,
        )
        self.DisableWarehouse = channel.unary_unary(
            "/io.openaristos.Bakplane/DisableWarehouse",
            request_serializer=bakplane_dot_bakplane__pb2.DisableWarehouseRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.DisableWarehouseResponse.FromString,
        )
        self.ApplyMasteringEngine = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyMasteringEngine",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyMasteringEngineRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyMasteringEngineResponse.FromString,
        )
        self.RetractMasteringEngine = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractMasteringEngine",
            request_serializer=bakplane_dot_bakplane__pb2.RetractMasteringEngineRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractMasteringEngineResponse.FromString,
        )
        self.ListMasteringEngine = channel.unary_unary(
            "/io.openaristos.Bakplane/ListMasteringEngine",
            request_serializer=bakplane_dot_bakplane__pb2.ListMasteringEngineRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListMasteringEngineResponse.FromString,
        )
        self.CreateIngestionSession = channel.unary_unary(
            "/io.openaristos.Bakplane/CreateIngestionSession",
            request_serializer=bakplane_dot_bakplane__pb2.CreateIngestionSessionRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.CreateIngestionSessionResponse.FromString,
        )
        self.ExecuteIngestionSession = channel.unary_unary(
            "/io.openaristos.Bakplane/ExecuteIngestionSession",
            request_serializer=bakplane_dot_bakplane__pb2.ExecuteIngestionSessionRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.IngestionSessionStatusResponse.FromString,
        )
        self.DeleteIngestionSession = channel.unary_unary(
            "/io.openaristos.Bakplane/DeleteIngestionSession",
            request_serializer=bakplane_dot_bakplane__pb2.DeleteIngestionSessionRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.DeleteIngestionSessionResponse.FromString,
        )
        self.GetIngestionSession = channel.unary_unary(
            "/io.openaristos.Bakplane/GetIngestionSession",
            request_serializer=bakplane_dot_bakplane__pb2.GetIngestionSessionRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.IngestionSessionStatusResponse.FromString,
        )
        self.MasterIngestionSession = channel.unary_unary(
            "/io.openaristos.Bakplane/MasterIngestionSession",
            request_serializer=bakplane_dot_bakplane__pb2.MasterIngestionSessionRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.MasterIngestionSessionResponse.FromString,
        )
        self.HaltIncrementalMastering = channel.unary_unary(
            "/io.openaristos.Bakplane/HaltIncrementalMastering",
            request_serializer=bakplane_dot_bakplane__pb2.HaltIncrementalMasteringRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.HaltIncrementalMasteringResponse.FromString,
        )
        self.ResumeIncrementalMastering = channel.unary_unary(
            "/io.openaristos.Bakplane/ResumeIncrementalMastering",
            request_serializer=bakplane_dot_bakplane__pb2.ResumeIncrementalMasteringRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ResumeIncrementalMasteringResponse.FromString,
        )
        self.PublishIngestionSessionResource = channel.unary_unary(
            "/io.openaristos.Bakplane/PublishIngestionSessionResource",
            request_serializer=bakplane_dot_bakplane__pb2.PublishIngestionSessionResourceRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.PublishIngestionSessionResourceResponse.FromString,
        )
        self.PublishIngestionSessionResourceStatus = channel.unary_unary(
            "/io.openaristos.Bakplane/PublishIngestionSessionResourceStatus",
            request_serializer=bakplane_dot_bakplane__pb2.PublishIngestionSessionResourceStatusRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.PublishIngestionSessionStatusResponse.FromString,
        )
        self.RetractPublishingSession = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractPublishingSession",
            request_serializer=bakplane_dot_bakplane__pb2.RetractPublishingSessionRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractPublishingSessionResponse.FromString,
        )
        self.RetractMasteringSession = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractMasteringSession",
            request_serializer=bakplane_dot_bakplane__pb2.RetractMasteringSessionRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractMasteringSessionResponse.FromString,
        )
        self.ResolveResourcePath = channel.unary_unary(
            "/io.openaristos.Bakplane/ResolveResourcePath",
            request_serializer=bakplane_dot_bakplane__pb2.ResolveResourcePathRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ResolveResourcePathResponse.FromString,
        )
        self.ListIngestionSessionArtifacts = channel.unary_unary(
            "/io.openaristos.Bakplane/ListIngestionSessionArtifacts",
            request_serializer=bakplane_dot_bakplane__pb2.ListIngestionSessionArtifactsRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListIngestionSessionArtifactsResponse.FromString,
        )
        self.ListIngestionSessions = channel.unary_unary(
            "/io.openaristos.Bakplane/ListIngestionSessions",
            request_serializer=bakplane_dot_bakplane__pb2.ListIngestionSessionsRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListIngestionSessionsResponse.FromString,
        )
        self.SessionPS = channel.unary_unary(
            "/io.openaristos.Bakplane/SessionPS",
            request_serializer=bakplane_dot_bakplane__pb2.PSRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.PSResponse.FromString,
        )
        self.ApplyPerspective = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyPerspective",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyPerspectiveRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyPerspectiveResponse.FromString,
        )
        self.RetractPerspective = channel.unary_unary(
            "/io.openaristos.Bakplane/RetractPerspective",
            request_serializer=bakplane_dot_bakplane__pb2.RetractPerspectiveRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.RetractPerspectiveResponse.FromString,
        )
        self.ListPerspective = channel.unary_unary(
            "/io.openaristos.Bakplane/ListPerspective",
            request_serializer=bakplane_dot_bakplane__pb2.ListPerspectiveRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListPerspectiveResponse.FromString,
        )
        self.HealthCheck = channel.unary_unary(
            "/io.openaristos.Bakplane/HealthCheck",
            request_serializer=bakplane_dot_bakplane__pb2.HealthCheckRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.HealthCheckResponse.FromString,
        )
        self.EntityCoverage = channel.unary_unary(
            "/io.openaristos.Bakplane/EntityCoverage",
            request_serializer=bakplane_dot_bakplane__pb2.EntityCoverageRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.EntityCoverageResponse.FromString,
        )
        self.ApplyApp = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyApp",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyAppRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyAppResponse.FromString,
        )
        self.ExecuteApp = channel.unary_unary(
            "/io.openaristos.Bakplane/ExecuteApp",
            request_serializer=bakplane_dot_bakplane__pb2.ExecuteAppRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ExecuteAppResponse.FromString,
        )
        self.ApplyDependency = channel.unary_unary(
            "/io.openaristos.Bakplane/ApplyDependency",
            request_serializer=bakplane_dot_bakplane__pb2.ApplyDependencyRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ApplyDependencyResponse.FromString,
        )
        self.ListApp = channel.unary_unary(
            "/io.openaristos.Bakplane/ListApp",
            request_serializer=bakplane_dot_bakplane__pb2.ListAppRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.ListAppResponse.FromString,
        )
        self.EmitEvent = channel.unary_unary(
            "/io.openaristos.Bakplane/EmitEvent",
            request_serializer=bakplane_dot_bakplane__pb2.EmitEventRequest.SerializeToString,
            response_deserializer=bakplane_dot_bakplane__pb2.EmitEventResponse.FromString,
        )


class BakplaneServicer(object):
    """
    Service for manging the lifecycle of ingestion, mastering, warehousing, and querying

    """

    def ApplyAdapter(self, request, context):
        """Apply an adapter, a new adapter is created, and an existing is updated, including all resources"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractAdapter(self, request, context):
        """Retract an adapter, removes all resources, ingestion sessions, then removes the adapter"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListAdapter(self, request, context):
        """List adapters by query definition"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def EnableAdapter(self, request, context):
        """Enable the adapter, by default the adapter is enabled, only if you had disabled will you need to run this"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DisableAdapter(self, request, context):
        """Disable the adapter, this is useful, when you want to keep your adapter around, but no longer need it"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ApplyResource(self, request, context):
        """Apply a resource, a new resource is created, a resource here may be independent from the adatper"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractResource(self, request, context):
        """Retract a resource"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListResources(self, request, context):
        """List resources by query definition"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def EnableResource(self, request, context):
        """Enable the resource, by default the resource is enabled, only if you had disabled will you need to run this"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def PreviewResource(self, request, context):
        """Preview the resource data, providing up to 100 rows as sample"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DisableResource(self, request, context):
        """Disable the resource, this is useful, when you want to keep your resource around, but no longer need it"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def MigrateResource(self, request, context):
        """Database migrations, including up, down, and getting a version, very powerful for managing schemas"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def MigrateAllResources(self, request, context):
        """Migrate all resources for a given adapter"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ResourceMigrationStatus(self, request, context):
        """Database migration status for a given adapter and warehouse provides the version it's running currently"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ResolveResource(self, request, context):
        """Resolve resource allows a user to request resource data in a warehouse (virtual or physical)"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ResolveResourceIntent(self, request, context):
        """Resolve resource intent, provides location where given resource is, connectivity information how to access it"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ResolveQueryResource(self, request, context):
        """Resolves a resource for querying purposes"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ResolveUniverse(self, request, context):
        """Resolves a traversal request"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ApplyLandingZone(self, request, context):
        """Apply an landing zone, a new landing zone is created, and an existing is updated"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractLandingZone(self, request, context):
        """Retract an landing zone, removes all data inside of landing zone, then removes the landing zone plugin"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListLandingZone(self, request, context):
        """List landing zones by query definition"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def EnableLandingZone(self, request, context):
        """Enable a single landing zone"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DisableLandingZone(self, request, context):
        """Disable a single landing zone, this will leave data intact, but disable the use of the plugin"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ApplyOntology(self, request, context):
        """Apply an ontology, a new ontology is created, and an existing is updated"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractOntology(self, request, context):
        """Retract an ontology, removes all entities and relationships inside of Dominus"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListOntology(self, request, context):
        """List all installed ontologies"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ApplyExecutionEngine(self, request, context):
        """Apply an execution engine, a new execution engine is created, and an existing is updated"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractExecutionEngine(self, request, context):
        """Retract an execution engine, this will disable all plugins that depend on it, such as adapters"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListExecutionEngine(self, request, context):
        """List execution engine by query definition"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def EnableExecutionEngine(self, request, context):
        """Enable a single execution engine"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DisableExecutionEngine(self, request, context):
        """Disable a single execution engine, this will disable all plugins that depend on it"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ApplyWarehouse(self, request, context):
        """Apply an warehouse, a new warehouse is created, and an existing is updated"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ApplyValidator(self, request, context):
        """Apply a custom validator into the system"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractValidator(self, request, context):
        """Retract an validator that should no longer be used"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListValidator(self, request, context):
        """List validator by query definition"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractWarehouse(self, request, context):
        """Retract an warehouse, this will disable all plugins that depend on it, such as adapters"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListWarehouse(self, request, context):
        """List warehouse by query definition"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def EnableWarehouse(self, request, context):
        """Enable a single warehouse"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DisableWarehouse(self, request, context):
        """Disable a single warehouse, this will disable all plugins that depend on it, including resources"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ApplyMasteringEngine(self, request, context):
        """Apply a mastering engine, a new mastering engine is created, and an existing is updated,
        the Dominus mastering engine is the only one supported at this time
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractMasteringEngine(self, request, context):
        """Retract a existing mastering engine"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListMasteringEngine(self, request, context):
        """List mastering engines by query definition"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def CreateIngestionSession(self, request, context):
        """Create an ingestion session, this is used to ingest bounded, and unbounded data, you may supply
        your own adapter parameters, and remember that this operates on a per adapter basis
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ExecuteIngestionSession(self, request, context):
        """Execute an ingestion session"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def DeleteIngestionSession(self, request, context):
        """Delete an ingestion session"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def GetIngestionSession(self, request, context):
        """Get the ingestion session status"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def MasterIngestionSession(self, request, context):
        """Execute an a batch mastering session, this informs Dominus to batch load the entities and relationships"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def HaltIncrementalMastering(self, request, context):
        """Halt incremental mastering for the given engines"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ResumeIncrementalMastering(self, request, context):
        """Halt incremental mastering for the given engines"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def PublishIngestionSessionResource(self, request, context):
        """Publish all asset artifacts produced by an ingestion session into one or more warehouses"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def PublishIngestionSessionResourceStatus(self, request, context):
        """Get the last known status of the ingestion session"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractPublishingSession(self, request, context):
        """Retract items created and published during the ingestion session"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractMasteringSession(self, request, context):
        """Retract items created and published during the ingestion session"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ResolveResourcePath(self, request, context):
        """Resolve a given path into a detailed resource location"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListIngestionSessionArtifacts(self, request, context):
        """Retrieve the ingestion session artifact locations"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListIngestionSessions(self, request, context):
        """List ingestion sessions over a period of time"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def SessionPS(self, request, context):
        """Lists running ingestion and publishing sessions"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ApplyPerspective(self, request, context):
        """Apply a perspective, a new perspective is created, and an existing is updated"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def RetractPerspective(self, request, context):
        """Retract an perspective, removes all entities related to the perspective, then removes the perspective"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListPerspective(self, request, context):
        """List perspectives by query definition"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def HealthCheck(self, request, context):
        """A basic health-check request"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def EntityCoverage(self, request, context):
        """An entity coverage endpoint"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ApplyApp(self, request, context):
        """Apply an application"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ExecuteApp(self, request, context):
        """Execute an application"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ApplyDependency(self, request, context):
        """Apply an application dependency"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def ListApp(self, request, context):
        """List applications"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")

    def EmitEvent(self, request, context):
        """Emit a custom event into the broker"""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details("Method not implemented!")
        raise NotImplementedError("Method not implemented!")


def add_BakplaneServicer_to_server(servicer, server):
    rpc_method_handlers = {
        "ApplyAdapter": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyAdapter,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyAdapterRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyAdapterResponse.SerializeToString,
        ),
        "RetractAdapter": grpc.unary_unary_rpc_method_handler(
            servicer.RetractAdapter,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractAdapterRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractAdapterResponse.SerializeToString,
        ),
        "ListAdapter": grpc.unary_unary_rpc_method_handler(
            servicer.ListAdapter,
            request_deserializer=bakplane_dot_bakplane__pb2.ListAdapterRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListAdapterResponse.SerializeToString,
        ),
        "EnableAdapter": grpc.unary_unary_rpc_method_handler(
            servicer.EnableAdapter,
            request_deserializer=bakplane_dot_bakplane__pb2.EnableAdapterRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.EnableAdapterResponse.SerializeToString,
        ),
        "DisableAdapter": grpc.unary_unary_rpc_method_handler(
            servicer.DisableAdapter,
            request_deserializer=bakplane_dot_bakplane__pb2.DisableAdapterRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.DisableAdapterResponse.SerializeToString,
        ),
        "ApplyResource": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyResource,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyResourceResponse.SerializeToString,
        ),
        "RetractResource": grpc.unary_unary_rpc_method_handler(
            servicer.RetractResource,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractResourceResponse.SerializeToString,
        ),
        "ListResources": grpc.unary_unary_rpc_method_handler(
            servicer.ListResources,
            request_deserializer=bakplane_dot_bakplane__pb2.ListResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListResourceResponse.SerializeToString,
        ),
        "EnableResource": grpc.unary_unary_rpc_method_handler(
            servicer.EnableResource,
            request_deserializer=bakplane_dot_bakplane__pb2.EnableResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.EnableResourceResponse.SerializeToString,
        ),
        "PreviewResource": grpc.unary_unary_rpc_method_handler(
            servicer.PreviewResource,
            request_deserializer=bakplane_dot_bakplane__pb2.PreviewResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.PreviewResourceResponse.SerializeToString,
        ),
        "DisableResource": grpc.unary_unary_rpc_method_handler(
            servicer.DisableResource,
            request_deserializer=bakplane_dot_bakplane__pb2.DisableResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.DisableResourceResponse.SerializeToString,
        ),
        "MigrateResource": grpc.unary_unary_rpc_method_handler(
            servicer.MigrateResource,
            request_deserializer=bakplane_dot_bakplane__pb2.MigrateResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.MigrateResourceResponse.SerializeToString,
        ),
        "MigrateAllResources": grpc.unary_unary_rpc_method_handler(
            servicer.MigrateAllResources,
            request_deserializer=bakplane_dot_bakplane__pb2.MigrateAllResourcesRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.MigrateAllResourcesResponse.SerializeToString,
        ),
        "ResourceMigrationStatus": grpc.unary_unary_rpc_method_handler(
            servicer.ResourceMigrationStatus,
            request_deserializer=bakplane_dot_bakplane__pb2.MigrationStatusRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.MigrateResourceResponse.SerializeToString,
        ),
        "ResolveResource": grpc.unary_stream_rpc_method_handler(
            servicer.ResolveResource,
            request_deserializer=bakplane_dot_bakplane__pb2.ResolveResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ResolveResourceResponse.SerializeToString,
        ),
        "ResolveResourceIntent": grpc.unary_unary_rpc_method_handler(
            servicer.ResolveResourceIntent,
            request_deserializer=bakplane_dot_bakplane__pb2.ResolveResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ResolveResourceIntentResponse.SerializeToString,
        ),
        "ResolveQueryResource": grpc.unary_unary_rpc_method_handler(
            servicer.ResolveQueryResource,
            request_deserializer=bakplane_dot_bakplane__pb2.ResolveQueryResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ResolveQueryResourceResponse.SerializeToString,
        ),
        "ResolveUniverse": grpc.unary_stream_rpc_method_handler(
            servicer.ResolveUniverse,
            request_deserializer=bakplane_dot_bakplane__pb2.ResolveUniverseRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ResolveUniverseGroupEntry.SerializeToString,
        ),
        "ApplyLandingZone": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyLandingZone,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyLandingZoneRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyLandingZoneResponse.SerializeToString,
        ),
        "RetractLandingZone": grpc.unary_unary_rpc_method_handler(
            servicer.RetractLandingZone,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractLandingZoneRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractLandingZoneResponse.SerializeToString,
        ),
        "ListLandingZone": grpc.unary_unary_rpc_method_handler(
            servicer.ListLandingZone,
            request_deserializer=bakplane_dot_bakplane__pb2.ListLandingZoneRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListLandingZoneResponse.SerializeToString,
        ),
        "EnableLandingZone": grpc.unary_unary_rpc_method_handler(
            servicer.EnableLandingZone,
            request_deserializer=bakplane_dot_bakplane__pb2.EnableLandingZoneRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.EnableLandingZoneResponse.SerializeToString,
        ),
        "DisableLandingZone": grpc.unary_unary_rpc_method_handler(
            servicer.DisableLandingZone,
            request_deserializer=bakplane_dot_bakplane__pb2.DisableLandingZoneRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.DisableLandingZoneResponse.SerializeToString,
        ),
        "ApplyOntology": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyOntology,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyOntologyRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyOntologyResponse.SerializeToString,
        ),
        "RetractOntology": grpc.unary_unary_rpc_method_handler(
            servicer.RetractOntology,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractOntologyRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractOntologyResponse.SerializeToString,
        ),
        "ListOntology": grpc.unary_unary_rpc_method_handler(
            servicer.ListOntology,
            request_deserializer=bakplane_dot_bakplane__pb2.ListOntologyRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListOntologyResponse.SerializeToString,
        ),
        "ApplyExecutionEngine": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyExecutionEngine,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyExecutionEngineRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyExecutionEngineResponse.SerializeToString,
        ),
        "RetractExecutionEngine": grpc.unary_unary_rpc_method_handler(
            servicer.RetractExecutionEngine,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractExecutionEngineRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractExecutionEngineResponse.SerializeToString,
        ),
        "ListExecutionEngine": grpc.unary_unary_rpc_method_handler(
            servicer.ListExecutionEngine,
            request_deserializer=bakplane_dot_bakplane__pb2.ListExecutionEngineRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListExecutionEngineResponse.SerializeToString,
        ),
        "EnableExecutionEngine": grpc.unary_unary_rpc_method_handler(
            servicer.EnableExecutionEngine,
            request_deserializer=bakplane_dot_bakplane__pb2.EnableExecutionEngineRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.EnableExecutionEngineResponse.SerializeToString,
        ),
        "DisableExecutionEngine": grpc.unary_unary_rpc_method_handler(
            servicer.DisableExecutionEngine,
            request_deserializer=bakplane_dot_bakplane__pb2.DisableExecutionEngineRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.DisableExecutionEngineResponse.SerializeToString,
        ),
        "ApplyWarehouse": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyWarehouse,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyWarehouseRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyWarehouseResponse.SerializeToString,
        ),
        "ApplyValidator": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyValidator,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyValidatorRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyValidatorResponse.SerializeToString,
        ),
        "RetractValidator": grpc.unary_unary_rpc_method_handler(
            servicer.RetractValidator,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractValidatorRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractValidatorResponse.SerializeToString,
        ),
        "ListValidator": grpc.unary_unary_rpc_method_handler(
            servicer.ListValidator,
            request_deserializer=bakplane_dot_bakplane__pb2.ListValidatorRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListValidatorResponse.SerializeToString,
        ),
        "RetractWarehouse": grpc.unary_unary_rpc_method_handler(
            servicer.RetractWarehouse,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractWarehouseRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractWarehouseResponse.SerializeToString,
        ),
        "ListWarehouse": grpc.unary_unary_rpc_method_handler(
            servicer.ListWarehouse,
            request_deserializer=bakplane_dot_bakplane__pb2.ListWarehouseRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListWarehouseResponse.SerializeToString,
        ),
        "EnableWarehouse": grpc.unary_unary_rpc_method_handler(
            servicer.EnableWarehouse,
            request_deserializer=bakplane_dot_bakplane__pb2.EnableWarehouseRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.EnableWarehouseResponse.SerializeToString,
        ),
        "DisableWarehouse": grpc.unary_unary_rpc_method_handler(
            servicer.DisableWarehouse,
            request_deserializer=bakplane_dot_bakplane__pb2.DisableWarehouseRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.DisableWarehouseResponse.SerializeToString,
        ),
        "ApplyMasteringEngine": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyMasteringEngine,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyMasteringEngineRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyMasteringEngineResponse.SerializeToString,
        ),
        "RetractMasteringEngine": grpc.unary_unary_rpc_method_handler(
            servicer.RetractMasteringEngine,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractMasteringEngineRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractMasteringEngineResponse.SerializeToString,
        ),
        "ListMasteringEngine": grpc.unary_unary_rpc_method_handler(
            servicer.ListMasteringEngine,
            request_deserializer=bakplane_dot_bakplane__pb2.ListMasteringEngineRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListMasteringEngineResponse.SerializeToString,
        ),
        "CreateIngestionSession": grpc.unary_unary_rpc_method_handler(
            servicer.CreateIngestionSession,
            request_deserializer=bakplane_dot_bakplane__pb2.CreateIngestionSessionRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.CreateIngestionSessionResponse.SerializeToString,
        ),
        "ExecuteIngestionSession": grpc.unary_unary_rpc_method_handler(
            servicer.ExecuteIngestionSession,
            request_deserializer=bakplane_dot_bakplane__pb2.ExecuteIngestionSessionRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.IngestionSessionStatusResponse.SerializeToString,
        ),
        "DeleteIngestionSession": grpc.unary_unary_rpc_method_handler(
            servicer.DeleteIngestionSession,
            request_deserializer=bakplane_dot_bakplane__pb2.DeleteIngestionSessionRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.DeleteIngestionSessionResponse.SerializeToString,
        ),
        "GetIngestionSession": grpc.unary_unary_rpc_method_handler(
            servicer.GetIngestionSession,
            request_deserializer=bakplane_dot_bakplane__pb2.GetIngestionSessionRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.IngestionSessionStatusResponse.SerializeToString,
        ),
        "MasterIngestionSession": grpc.unary_unary_rpc_method_handler(
            servicer.MasterIngestionSession,
            request_deserializer=bakplane_dot_bakplane__pb2.MasterIngestionSessionRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.MasterIngestionSessionResponse.SerializeToString,
        ),
        "HaltIncrementalMastering": grpc.unary_unary_rpc_method_handler(
            servicer.HaltIncrementalMastering,
            request_deserializer=bakplane_dot_bakplane__pb2.HaltIncrementalMasteringRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.HaltIncrementalMasteringResponse.SerializeToString,
        ),
        "ResumeIncrementalMastering": grpc.unary_unary_rpc_method_handler(
            servicer.ResumeIncrementalMastering,
            request_deserializer=bakplane_dot_bakplane__pb2.ResumeIncrementalMasteringRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ResumeIncrementalMasteringResponse.SerializeToString,
        ),
        "PublishIngestionSessionResource": grpc.unary_unary_rpc_method_handler(
            servicer.PublishIngestionSessionResource,
            request_deserializer=bakplane_dot_bakplane__pb2.PublishIngestionSessionResourceRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.PublishIngestionSessionResourceResponse.SerializeToString,
        ),
        "PublishIngestionSessionResourceStatus": grpc.unary_unary_rpc_method_handler(
            servicer.PublishIngestionSessionResourceStatus,
            request_deserializer=bakplane_dot_bakplane__pb2.PublishIngestionSessionResourceStatusRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.PublishIngestionSessionStatusResponse.SerializeToString,
        ),
        "RetractPublishingSession": grpc.unary_unary_rpc_method_handler(
            servicer.RetractPublishingSession,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractPublishingSessionRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractPublishingSessionResponse.SerializeToString,
        ),
        "RetractMasteringSession": grpc.unary_unary_rpc_method_handler(
            servicer.RetractMasteringSession,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractMasteringSessionRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractMasteringSessionResponse.SerializeToString,
        ),
        "ResolveResourcePath": grpc.unary_unary_rpc_method_handler(
            servicer.ResolveResourcePath,
            request_deserializer=bakplane_dot_bakplane__pb2.ResolveResourcePathRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ResolveResourcePathResponse.SerializeToString,
        ),
        "ListIngestionSessionArtifacts": grpc.unary_unary_rpc_method_handler(
            servicer.ListIngestionSessionArtifacts,
            request_deserializer=bakplane_dot_bakplane__pb2.ListIngestionSessionArtifactsRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListIngestionSessionArtifactsResponse.SerializeToString,
        ),
        "ListIngestionSessions": grpc.unary_unary_rpc_method_handler(
            servicer.ListIngestionSessions,
            request_deserializer=bakplane_dot_bakplane__pb2.ListIngestionSessionsRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListIngestionSessionsResponse.SerializeToString,
        ),
        "SessionPS": grpc.unary_unary_rpc_method_handler(
            servicer.SessionPS,
            request_deserializer=bakplane_dot_bakplane__pb2.PSRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.PSResponse.SerializeToString,
        ),
        "ApplyPerspective": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyPerspective,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyPerspectiveRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyPerspectiveResponse.SerializeToString,
        ),
        "RetractPerspective": grpc.unary_unary_rpc_method_handler(
            servicer.RetractPerspective,
            request_deserializer=bakplane_dot_bakplane__pb2.RetractPerspectiveRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.RetractPerspectiveResponse.SerializeToString,
        ),
        "ListPerspective": grpc.unary_unary_rpc_method_handler(
            servicer.ListPerspective,
            request_deserializer=bakplane_dot_bakplane__pb2.ListPerspectiveRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListPerspectiveResponse.SerializeToString,
        ),
        "HealthCheck": grpc.unary_unary_rpc_method_handler(
            servicer.HealthCheck,
            request_deserializer=bakplane_dot_bakplane__pb2.HealthCheckRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.HealthCheckResponse.SerializeToString,
        ),
        "EntityCoverage": grpc.unary_unary_rpc_method_handler(
            servicer.EntityCoverage,
            request_deserializer=bakplane_dot_bakplane__pb2.EntityCoverageRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.EntityCoverageResponse.SerializeToString,
        ),
        "ApplyApp": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyApp,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyAppRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyAppResponse.SerializeToString,
        ),
        "ExecuteApp": grpc.unary_unary_rpc_method_handler(
            servicer.ExecuteApp,
            request_deserializer=bakplane_dot_bakplane__pb2.ExecuteAppRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ExecuteAppResponse.SerializeToString,
        ),
        "ApplyDependency": grpc.unary_unary_rpc_method_handler(
            servicer.ApplyDependency,
            request_deserializer=bakplane_dot_bakplane__pb2.ApplyDependencyRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ApplyDependencyResponse.SerializeToString,
        ),
        "ListApp": grpc.unary_unary_rpc_method_handler(
            servicer.ListApp,
            request_deserializer=bakplane_dot_bakplane__pb2.ListAppRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.ListAppResponse.SerializeToString,
        ),
        "EmitEvent": grpc.unary_unary_rpc_method_handler(
            servicer.EmitEvent,
            request_deserializer=bakplane_dot_bakplane__pb2.EmitEventRequest.FromString,
            response_serializer=bakplane_dot_bakplane__pb2.EmitEventResponse.SerializeToString,
        ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
        "io.openaristos.Bakplane", rpc_method_handlers
    )
    server.add_generic_rpc_handlers((generic_handler,))


# This class is part of an EXPERIMENTAL API.
class Bakplane(object):
    """
    Service for manging the lifecycle of ingestion, mastering, warehousing, and querying

    """

    @staticmethod
    def ApplyAdapter(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyAdapter",
            bakplane_dot_bakplane__pb2.ApplyAdapterRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyAdapterResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractAdapter(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractAdapter",
            bakplane_dot_bakplane__pb2.RetractAdapterRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractAdapterResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListAdapter(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListAdapter",
            bakplane_dot_bakplane__pb2.ListAdapterRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListAdapterResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def EnableAdapter(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/EnableAdapter",
            bakplane_dot_bakplane__pb2.EnableAdapterRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.EnableAdapterResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DisableAdapter(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/DisableAdapter",
            bakplane_dot_bakplane__pb2.DisableAdapterRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.DisableAdapterResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ApplyResource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyResource",
            bakplane_dot_bakplane__pb2.ApplyResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractResource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractResource",
            bakplane_dot_bakplane__pb2.RetractResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListResources(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListResources",
            bakplane_dot_bakplane__pb2.ListResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def EnableResource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/EnableResource",
            bakplane_dot_bakplane__pb2.EnableResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.EnableResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def PreviewResource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/PreviewResource",
            bakplane_dot_bakplane__pb2.PreviewResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.PreviewResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DisableResource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/DisableResource",
            bakplane_dot_bakplane__pb2.DisableResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.DisableResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def MigrateResource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/MigrateResource",
            bakplane_dot_bakplane__pb2.MigrateResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.MigrateResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def MigrateAllResources(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/MigrateAllResources",
            bakplane_dot_bakplane__pb2.MigrateAllResourcesRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.MigrateAllResourcesResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ResourceMigrationStatus(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ResourceMigrationStatus",
            bakplane_dot_bakplane__pb2.MigrationStatusRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.MigrateResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ResolveResource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_stream(
            request,
            target,
            "/io.openaristos.Bakplane/ResolveResource",
            bakplane_dot_bakplane__pb2.ResolveResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ResolveResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ResolveResourceIntent(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ResolveResourceIntent",
            bakplane_dot_bakplane__pb2.ResolveResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ResolveResourceIntentResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ResolveQueryResource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ResolveQueryResource",
            bakplane_dot_bakplane__pb2.ResolveQueryResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ResolveQueryResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ResolveUniverse(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_stream(
            request,
            target,
            "/io.openaristos.Bakplane/ResolveUniverse",
            bakplane_dot_bakplane__pb2.ResolveUniverseRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ResolveUniverseGroupEntry.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ApplyLandingZone(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyLandingZone",
            bakplane_dot_bakplane__pb2.ApplyLandingZoneRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyLandingZoneResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractLandingZone(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractLandingZone",
            bakplane_dot_bakplane__pb2.RetractLandingZoneRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractLandingZoneResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListLandingZone(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListLandingZone",
            bakplane_dot_bakplane__pb2.ListLandingZoneRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListLandingZoneResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def EnableLandingZone(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/EnableLandingZone",
            bakplane_dot_bakplane__pb2.EnableLandingZoneRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.EnableLandingZoneResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DisableLandingZone(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/DisableLandingZone",
            bakplane_dot_bakplane__pb2.DisableLandingZoneRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.DisableLandingZoneResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ApplyOntology(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyOntology",
            bakplane_dot_bakplane__pb2.ApplyOntologyRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyOntologyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractOntology(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractOntology",
            bakplane_dot_bakplane__pb2.RetractOntologyRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractOntologyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListOntology(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListOntology",
            bakplane_dot_bakplane__pb2.ListOntologyRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListOntologyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ApplyExecutionEngine(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyExecutionEngine",
            bakplane_dot_bakplane__pb2.ApplyExecutionEngineRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyExecutionEngineResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractExecutionEngine(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractExecutionEngine",
            bakplane_dot_bakplane__pb2.RetractExecutionEngineRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractExecutionEngineResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListExecutionEngine(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListExecutionEngine",
            bakplane_dot_bakplane__pb2.ListExecutionEngineRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListExecutionEngineResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def EnableExecutionEngine(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/EnableExecutionEngine",
            bakplane_dot_bakplane__pb2.EnableExecutionEngineRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.EnableExecutionEngineResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DisableExecutionEngine(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/DisableExecutionEngine",
            bakplane_dot_bakplane__pb2.DisableExecutionEngineRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.DisableExecutionEngineResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ApplyWarehouse(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyWarehouse",
            bakplane_dot_bakplane__pb2.ApplyWarehouseRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyWarehouseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ApplyValidator(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyValidator",
            bakplane_dot_bakplane__pb2.ApplyValidatorRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyValidatorResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractValidator(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractValidator",
            bakplane_dot_bakplane__pb2.RetractValidatorRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractValidatorResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListValidator(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListValidator",
            bakplane_dot_bakplane__pb2.ListValidatorRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListValidatorResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractWarehouse(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractWarehouse",
            bakplane_dot_bakplane__pb2.RetractWarehouseRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractWarehouseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListWarehouse(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListWarehouse",
            bakplane_dot_bakplane__pb2.ListWarehouseRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListWarehouseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def EnableWarehouse(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/EnableWarehouse",
            bakplane_dot_bakplane__pb2.EnableWarehouseRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.EnableWarehouseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DisableWarehouse(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/DisableWarehouse",
            bakplane_dot_bakplane__pb2.DisableWarehouseRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.DisableWarehouseResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ApplyMasteringEngine(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyMasteringEngine",
            bakplane_dot_bakplane__pb2.ApplyMasteringEngineRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyMasteringEngineResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractMasteringEngine(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractMasteringEngine",
            bakplane_dot_bakplane__pb2.RetractMasteringEngineRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractMasteringEngineResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListMasteringEngine(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListMasteringEngine",
            bakplane_dot_bakplane__pb2.ListMasteringEngineRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListMasteringEngineResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def CreateIngestionSession(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/CreateIngestionSession",
            bakplane_dot_bakplane__pb2.CreateIngestionSessionRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.CreateIngestionSessionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ExecuteIngestionSession(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ExecuteIngestionSession",
            bakplane_dot_bakplane__pb2.ExecuteIngestionSessionRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.IngestionSessionStatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def DeleteIngestionSession(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/DeleteIngestionSession",
            bakplane_dot_bakplane__pb2.DeleteIngestionSessionRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.DeleteIngestionSessionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def GetIngestionSession(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/GetIngestionSession",
            bakplane_dot_bakplane__pb2.GetIngestionSessionRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.IngestionSessionStatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def MasterIngestionSession(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/MasterIngestionSession",
            bakplane_dot_bakplane__pb2.MasterIngestionSessionRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.MasterIngestionSessionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def HaltIncrementalMastering(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/HaltIncrementalMastering",
            bakplane_dot_bakplane__pb2.HaltIncrementalMasteringRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.HaltIncrementalMasteringResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ResumeIncrementalMastering(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ResumeIncrementalMastering",
            bakplane_dot_bakplane__pb2.ResumeIncrementalMasteringRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ResumeIncrementalMasteringResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def PublishIngestionSessionResource(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/PublishIngestionSessionResource",
            bakplane_dot_bakplane__pb2.PublishIngestionSessionResourceRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.PublishIngestionSessionResourceResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def PublishIngestionSessionResourceStatus(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/PublishIngestionSessionResourceStatus",
            bakplane_dot_bakplane__pb2.PublishIngestionSessionResourceStatusRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.PublishIngestionSessionStatusResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractPublishingSession(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractPublishingSession",
            bakplane_dot_bakplane__pb2.RetractPublishingSessionRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractPublishingSessionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractMasteringSession(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractMasteringSession",
            bakplane_dot_bakplane__pb2.RetractMasteringSessionRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractMasteringSessionResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ResolveResourcePath(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ResolveResourcePath",
            bakplane_dot_bakplane__pb2.ResolveResourcePathRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ResolveResourcePathResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListIngestionSessionArtifacts(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListIngestionSessionArtifacts",
            bakplane_dot_bakplane__pb2.ListIngestionSessionArtifactsRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListIngestionSessionArtifactsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListIngestionSessions(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListIngestionSessions",
            bakplane_dot_bakplane__pb2.ListIngestionSessionsRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListIngestionSessionsResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def SessionPS(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/SessionPS",
            bakplane_dot_bakplane__pb2.PSRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.PSResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ApplyPerspective(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyPerspective",
            bakplane_dot_bakplane__pb2.ApplyPerspectiveRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyPerspectiveResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def RetractPerspective(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/RetractPerspective",
            bakplane_dot_bakplane__pb2.RetractPerspectiveRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.RetractPerspectiveResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListPerspective(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListPerspective",
            bakplane_dot_bakplane__pb2.ListPerspectiveRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListPerspectiveResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def HealthCheck(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/HealthCheck",
            bakplane_dot_bakplane__pb2.HealthCheckRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.HealthCheckResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def EntityCoverage(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/EntityCoverage",
            bakplane_dot_bakplane__pb2.EntityCoverageRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.EntityCoverageResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ApplyApp(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyApp",
            bakplane_dot_bakplane__pb2.ApplyAppRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyAppResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ExecuteApp(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ExecuteApp",
            bakplane_dot_bakplane__pb2.ExecuteAppRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ExecuteAppResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ApplyDependency(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ApplyDependency",
            bakplane_dot_bakplane__pb2.ApplyDependencyRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ApplyDependencyResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def ListApp(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/ListApp",
            bakplane_dot_bakplane__pb2.ListAppRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.ListAppResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )

    @staticmethod
    def EmitEvent(
        request,
        target,
        options=(),
        channel_credentials=None,
        call_credentials=None,
        insecure=False,
        compression=None,
        wait_for_ready=None,
        timeout=None,
        metadata=None,
    ):
        return grpc.experimental.unary_unary(
            request,
            target,
            "/io.openaristos.Bakplane/EmitEvent",
            bakplane_dot_bakplane__pb2.EmitEventRequest.SerializeToString,
            bakplane_dot_bakplane__pb2.EmitEventResponse.FromString,
            options,
            channel_credentials,
            insecure,
            call_credentials,
            compression,
            wait_for_ready,
            timeout,
            metadata,
        )



import concurrent.futures
import datetime
import os
import tempfile
import typing
from abc import abstractmethod, ABC
from typing import IO

from google.protobuf.internal.encoder import _VarintBytes
from sortedcontainers import SortedSet

from bakplane.bakplane_pb2 import ErrorEntry
from bakplane.ingestion.landing_zone import LandingZoneFactory
from .models import (
    Asset,
    Entity,
    Relationship,
    IngestionSessionContext,
    Error,
    Descriptor,
)

PARTITION_NUM = os.cpu_count()
KEY_SEPARATOR = "::"


def reduce_pair(
    p1: typing.Tuple[datetime.datetime, datetime.datetime],
    p2: typing.Tuple[datetime.datetime, datetime.datetime],
):
    a, b = p1
    c, d = p2

    if a == c:
        if b < d:
            return (a, d), None
        elif b > d:
            return (a, b), None
        else:
            raise RuntimeError("Not allowed")
    elif a < c:
        if b == d:
            return (a, b), None
        elif b < d:
            if b < c:
                return (a, b), (c, d)
            elif b > c:
                return (a, d), None
            return (a, d), None
        else:
            return (a, b), None


def reduce_intervals(s: SortedSet) -> SortedSet:
    if not s:
        raise RuntimeError("Input cannot be none.")

    if len(s) == 1:
        return s

    done = SortedSet()

    last = s[0]

    for i in range(1, len(s)):
        p1 = s[i]
        i1, i2 = reduce_pair(last, p1)

        if i2 is not None:
            done.add(i1)
            done.add(i2)

            last = i2
        else:
            last = i1

    done.add(last)
    return done


class Emitter(ABC):
    @abstractmethod
    def emit_assets(self, assets: typing.List[Asset]):
        pass

    @abstractmethod
    def emit_entities(self, entities: typing.List[Entity]):
        pass

    @abstractmethod
    def emit_relationships(self, relationships: typing.List[Relationship]):
        pass

    @abstractmethod
    def emit_errors(self, errors: typing.List[Error]):
        pass


class RelationshipReduceMap:
    source_identity_uid: int
    target_identity_uid: int
    relationship_type: str
    effective_dating: SortedSet

    def __init__(self, r: Relationship):
        self.relationship_type = r.relationship_type
        self.target_identity_uid = r.target_identity_uid
        self.source_identity_uid = r.source_identity_uid
        self.effective_dating = SortedSet()

        self.append(r)

    def append(self, r: Relationship):
        if (
            r.source_identity_uid != self.source_identity_uid
            or r.target_identity_uid != self.target_identity_uid
            or r.relationship_type != self.relationship_type
        ):
            raise RuntimeError(
                "Invalid relationship identity for this reducer."
            )

        for e in r.effective_dating:
            self.effective_dating.add((e[0], e[1]))

    def reduce(self) -> Relationship:
        reduced_intervals = []

        for interval in reduce_intervals(self.effective_dating):
            start, end = interval

            reduced_intervals.append((start, end))

        return Relationship(
            relationship_type=self.relationship_type,
            source_identity_uid=self.source_identity_uid,
            target_identity_uid=self.target_identity_uid,
            effective_dating=reduced_intervals,
        )


class EntityReduceMap:
    def __init__(self, entity: Entity):
        self.identity = entity.identity
        self.values = {}

        self.append(entity)

    def append(self, entity: Entity):
        if entity.identity.uid != self.identity.uid:
            raise RuntimeError(
                f"Entity mismatch: expected entity with identity `{entity.identity.uid}`."
            )

        for d in entity.descriptors:
            self.append_descriptor(d)

    def append_descriptor(self, d: Descriptor):
        key = d.attribute + KEY_SEPARATOR + d.value

        if key not in self.values:
            self.values[key] = SortedSet()

        for dt in d.effective_dating:
            self.values[key].add((dt[0], dt[1]))

    def reduce(self) -> Entity:
        descriptors: typing.List[Descriptor] = []

        for key, v in self.values.items():
            attr, val = key.split(KEY_SEPARATOR)

            effective_dating = []

            for interval in reduce_intervals(v):
                start, end = interval

                effective_dating.append((start, end))

            descriptors.append(
                Descriptor(
                    attribute=attr,
                    value=val,
                    effective_dating=effective_dating,
                )
            )

        self.values = None
        return Entity(identity=self.identity, descriptors=descriptors)


class DefaultEmitter(Emitter):
    def __init__(
        self,
        context: IngestionSessionContext,
        asset_num_sharding=8,
        entities_num_sharding=8,
        rels_num_sharding=8,
        csv_separator: str = "|",
        null_value: str = "NULL",
    ):

        if (
            asset_num_sharding < 1
            or entities_num_sharding < 1
            or rels_num_sharding < 1
        ):
            raise RuntimeError("Invalid requested number of shards.")

        self.ctx = context
        self.fd_map: typing.Dict[str, IO] = {}

        self.entities_lz = LandingZoneFactory.build_from_uri(
            context.entities_output_location
        )

        self.assets_lz = LandingZoneFactory.build_from_uri(
            context.assets_output_location
        )

        self.relationships_lz = LandingZoneFactory.build_from_uri(
            context.relationships_output_location
        )

        self.errors_lz = LandingZoneFactory.build_from_uri(
            context.errors_output_location
        )

        self.asset_num_sharding = asset_num_sharding
        self.entities_num_sharding = entities_num_sharding
        self.rels_num_sharding = rels_num_sharding

        self.entity_map = {}
        self.rel_map = {}

        self.csv_separator = csv_separator
        self.null_value = null_value

    def __get_entities_fd(self, entity: Entity) -> IO:
        key = "entities"

        if self.entities_num_sharding > 1:
            shard = entity.identity.uid % self.entities_num_sharding
            key += f"___{shard}_of_{self.entities_num_sharding}_"

        if key not in self.fd_map:
            self.fd_map[key] = tempfile.NamedTemporaryFile(
                mode="w+b", delete=False, prefix=key
            )
        return self.fd_map[key]

    def __get_errors_fd(self, error: ErrorEntry) -> IO:
        if "errors" not in self.fd_map:
            self.fd_map["errors"] = tempfile.NamedTemporaryFile(
                mode="w+b", delete=False, prefix="errors"
            )
        return self.fd_map["errors"]

    def __get_relationships_fd(self, relationship: Relationship) -> IO:
        key = "relationships"

        if self.rels_num_sharding > 1:
            shard = relationship.source_identity_uid % self.rels_num_sharding
            key += f"___{shard}_of_{self.rels_num_sharding}_"

        if key not in self.fd_map:
            self.fd_map[key] = tempfile.NamedTemporaryFile(
                mode="w+b", delete=False, prefix=key
            )
        return self.fd_map[key]

    def __get_assets_fd(self, asset: Asset) -> IO:
        key = asset.resource_code

        if self.asset_num_sharding > 1:
            shard = asset.payload_hash % self.asset_num_sharding
            key += f"___{shard}_of_{self.asset_num_sharding}_"

        if key not in self.fd_map:
            self.fd_map[key] = tempfile.NamedTemporaryFile(
                mode="w+", delete=False, prefix=key
            )
        return self.fd_map[key]

    def __write_entity(self, entity: Entity):
        if entity.identity.uid not in self.entity_map:
            self.entity_map[entity.identity.uid] = EntityReduceMap(entity)
        else:
            self.entity_map[entity.identity.uid].append(entity)

    def __write_error(self, error: ErrorEntry):
        f = self.__get_errors_fd(error)
        p = error.to_proto()

        f.write(_VarintBytes(p.ByteSize()))
        f.write(p.SerializeToString())

    def __write_relationship(self, relationship: Relationship):
        if relationship.identity not in self.rel_map:
            self.rel_map[relationship.identity] = RelationshipReduceMap(
                relationship
            )
        else:
            self.rel_map[relationship.identity].append(relationship)

    def __write_asset(self, asset: Asset):
        f = self.__get_assets_fd(asset)
        f.write(
            asset.to_csv_entry(self.ctx, self.csv_separator, self.null_value)
            + "\n"
        )

    def emit_entity(self, entity: Entity):
        self.__write_entity(entity)

    def emit_asset(self, asset: Asset):
        self.ctx.validate_asset(asset)
        return self.__write_asset(asset)

    def emit_relationship(self, relationship: Relationship):
        return self.__write_relationship(relationship)

    def emit_error(self, err: Error):
        return self.__write_error(err)

    def emit_entities(self, entities: typing.List[Entity]):
        if entities is None or len(entities) <= 0:
            raise ValueError("Entities cannot be null or empty.")

        for e in entities:
            self.__write_entity(e)

    def emit_relationships(self, relationships: typing.List[Relationship]):
        if relationships is None or len(relationships) <= 0:
            raise ValueError("Relationships cannot be null or empty.")

        for r in relationships:
            self.__write_relationship(r)

    def emit_errors(self, errors: typing.List[Error]):
        if errors is None or len(errors) <= 0:
            raise ValueError("Errors cannot be null or empty.")

        for e in errors:
            self.__write_error(e)

    def emit_assets(self, assets: typing.List[Asset]):
        if assets is None or len(assets) <= 0:
            raise ValueError("Assets cannot be null or empty.")

        for a in assets:
            self.ctx.validate_asset(a)
            self.__write_asset(a)

    def reduce_entities(self):
        for k, e in self.entity_map.items():
            entity = e.reduce()

            f = self.__get_entities_fd(entity)
            p = entity.to_proto()

            f.write(_VarintBytes(p.ByteSize()))
            f.write(p.SerializeToString())

    def reduce_relationships(self):
        for k, r in self.rel_map.items():
            relationship = r.reduce()

            f = self.__get_relationships_fd(relationship)
            p = relationship.to_proto()

            f.write(_VarintBytes(p.ByteSize()))
            f.write(p.SerializeToString())

    def close(self):
        self.reduce_entities()
        self.reduce_relationships()

        for k, v in self.fd_map.items():
            v.close()

            if k.startswith("entities"):
                self.entities_lz.upload(
                    v.name,
                    self.ctx.entities_output_location,
                    ".proto.gz",
                    prefix="entities",
                )

            elif k.startswith("relationships"):
                self.relationships_lz.upload(
                    v.name,
                    self.ctx.relationships_output_location,
                    ".proto.gz",
                    prefix="relationships",
                )

            elif k == "errors":
                self.errors_lz.upload(
                    v.name,
                    self.ctx.errors_output_location,
                    ".proto.gz",
                    prefix="errors",
                )
            else:
                asset_code = k
                if self.asset_num_sharding > 1:
                    asset_code = asset_code.split("___")[0]
                    self.assets_lz.upload(
                        v.name,
                        self.ctx.assets_output_location + asset_code + "/",
                        ".csv.gz",
                    )



from stdnum.gb import sedol
from stdnum import isin, cusip


def is_valid_cusip(idvalue):
    try:
        if len(idvalue) == 8:
            idvalue = idvalue + cusip.calc_check_digit(idvalue)
            is_valid = cusip.is_valid(idvalue)
        else:
            is_valid = cusip.is_valid(idvalue)
    except:
        is_valid = False
    return is_valid


def is_valid_sedol(idvalue):
    try:
        if len(idvalue) == 6:
            idvalue = idvalue + sedol.calc_check_digit(idvalue)
            is_valid = sedol.is_valid(idvalue)
        else:
            is_valid = sedol.is_valid(idvalue)
    except:
        is_valid = False
    return is_valid


def is_valid_isin(idvalue):
    try:
        if len(idvalue) == 11:
            idvalue = idvalue + isin.calc_check_digit(idvalue)
            is_valid = isin.is_valid(idvalue)
        else:
            is_valid = isin.is_valid(idvalue)
    except:
        is_valid = False
    return is_valid


def standardize_cusip(id_value):
    if cusip.is_valid(id_value):
        return id_value
    else:
        return id_value + cusip.calc_check_digit(id_value)


def standardize_sedol(id_value):
    if sedol.is_valid(id_value):
        return id_value
    else:
        return id_value + sedol.calc_check_digit(id_value)


def standardize_isin(id_value):
    if isin.is_valid(id_value):
        return id_value
    else:
        return id_value + isin.calc_check_digit(id_value)
